@@ -214,8 +214,8 @@ where
                             // Retry from the beginning.
                             continue;
                         }
-                        // Somebody else (a future containing `get_or_insert_with`/
-                        // `get_or_try_insert_with`) has been aborted.
+                        // Somebody else (a future containing `get_with`/`try_get_with`)
+                        // has been aborted.
                         WaiterValue::EnclosingFutureAborted => {
                             retries += 1;
                             panic_if_retry_exhausted_for_aborting(retries, MAX_RETRIES);
@@ -235,8 +235,8 @@ where
 
     #[inline]
     pub(crate) fn remove_waiter(&self, key: &Arc<K>, type_id: TypeId) {
-        let key = Arc::clone(key);
-        self.waiters.remove(&(key, type_id));
+        let (cht_key, hash) = self.cht_key_hash(key, type_id);
+        self.waiters.remove(&cht_key, hash);
     }
 
     #[inline]
@@ -246,11 +246,16 @@ where
         type_id: TypeId,
         waiter: &Waiter<V>,
     ) -> Option<Waiter<V>> {
-        let key = Arc::clone(key);
+        let (cht_key, hash) = self.cht_key_hash(key, type_id);
         let waiter = TrioArc::clone(waiter);
+        self.waiters.insert_if_not_present(cht_key, hash, waiter)
+    }
 
-        self.waiters
-            .insert_with_or_modify((key, type_id), || waiter, |_, w| TrioArc::clone(w))
+    #[inline]
+    fn cht_key_hash(&self, key: &Arc<K>, type_id: TypeId) -> ((Arc<K>, TypeId), u64) {
+        let cht_key = (Arc::clone(key), type_id);
+        let hash = self.waiters.hash(&cht_key);
+        (cht_key, hash)
     }
 }
 
@@ -268,8 +273,8 @@ fn panic_if_retry_exhausted_for_aborting(retries: usize, max: usize) {
     if retries >= max {
         panic!(
             "Too many retries. Tried to read the return value from the `init` future \
-    but failed {} times. Maybe the future containing `get_or_insert_with`/\
-    `get_or_try_insert_with` kept being aborted?",
+    but failed {} times. Maybe the future containing `get_with`/`try_get_with` \
+    kept being aborted?",
             retries
         );
     }
